@@ -5,7 +5,7 @@ BEGIN {
 
 # ABSTRACT: Reduce & analyze a 2 person matrix game
 
-our $VERSION = '0.0101';
+our $VERSION = '0.02';
 
 use strict;
 use warnings;
@@ -14,6 +14,7 @@ use Data::Dumper;
 use Algorithm::Combinatorics qw( variations_with_repetition );
 use List::Util qw( max );
 use List::MoreUtils qw( all indexes each_array );
+use Math::Calculus::Differentiate;
 
 
 
@@ -21,32 +22,45 @@ sub new {
     my $class = shift;
     my %args = @_;
     my $self = {
-        1 => $args{1} || { 1 => [1,0], 2 => [0,1] },
-        2 => $args{2} || { 1 => [1,0], 2 => [0,1] }
+        1 => $args{1} || {
+            strategy    => { 1 => [1,0], 2 => [0,1] },
+            payoff      => undef,
+        },
+        2 => $args{2} || {
+            strategy    => { 1 => [1,0], 2 => [0,1] },
+            payoff      => undef,
+        },
     };
     bless $self, $class;
+    $self->_init;
     return $self;
 }
 
+sub _init {
+    my $self = shift;
+#    my ($player, $opponent) = ($self->{1}{strategy}, $self->{2}{strategy});
+}
+
 
 sub reduce {
     my $self = shift;
 
     # Set the players.
     my ($player, $opponent) = @_;
-    ($player, $opponent) = ($self->{$player}, $self->{$opponent});
+    ($player, $opponent) = ($self->{$player}{strategy}, $self->{$opponent}{strategy});
 
     # Declare the bucket of "X given Y" strategy pair utilities.
     my $utility = {};
     my $metric  = {};
 
     # Evaluate pairs of strategies.
-    my $iter = variations_with_repetition([keys %$player], 2);
+    my $iter = variations_with_repetition([ keys %$player ], 2);
     while (my $v = $iter->next) {
         # Skip "X|X" pairs.
+        # XXX Only need to inspect combinations and flag relative utility.
         next if $v->[0] eq $v->[1];
 
-        # Inspect each stategy utility.
+        # Inspect each strategy utility.
         #warn join(', ', @$v), "\n";
         for my $i (0 .. @$v - 1) {
             # Only consider defined utilities.
@@ -104,7 +118,7 @@ sub nash {
     my $self = shift;
 
     # Convenience:
-    my ($player, $opponent) = ($self->{1}, $self->{2});
+    my ($player, $opponent) = ($self->{1}{strategy}, $self->{2}{strategy});
 
     # Inspect each player item for best strategy.
     my %x; # Max utility indexes.
@@ -135,6 +149,9 @@ sub nash {
     my @ystrat = sort { $a <=> $b } keys %y;
     my $estrat = each_array(@xstrat, @ystrat);
     while ( my ($xs, $ys) = $estrat->() ) {
+        # Skip non-strategies.
+        next if $xs eq 'payoff' || $ys eq 'payoff';
+
         #warn "xs:'@{$x{$xs}}'\n";
         #warn "ys:'@{$y{$ys}}'\n";
 
@@ -154,6 +171,77 @@ sub nash {
     return $nash;
 }
 
+
+sub payoff {
+    my $self = shift;
+
+    my @payoff;
+
+    for my $player (sort keys %$self) {
+        my @equation;
+        my @pinverse;
+        my @qinverse;
+
+        # Compute the payoff equation components.
+        for my $strat (sort keys %{ $self->{$player}{strategy} }) {
+            my $pinverse = '(1';
+            my $qinverse = '(1';
+            my $i = 0;
+            for my $util (@{ $self->{$player}{strategy}{$strat} }) {
+                $i++;
+                push @equation, "$util*p$strat*q$i";
+                $pinverse .= ' - p' . $i if $i <= $strat;
+                $qinverse .= ' - q' . $i if $i <= $strat;
+            }
+            $pinverse .= ')';
+            $qinverse .= ')';
+            push @pinverse, $pinverse;
+            push @qinverse, $qinverse;
+        }
+#warn "Player $player payoff: ", join(' + ', @equation), "\n\n";
+
+        # The last is unused. TODO Fix this with a correct condition, above.
+        pop @pinverse;
+        pop @qinverse;
+
+        # Substitute all the non-initial vars with (1 - ...
+        my $i = @pinverse + 1;
+        for my $inv (reverse @pinverse) {
+            @equation = grep { /p$i/ ? s/p$i/$inv/ : $_ } @equation;
+            $i--;
+        }
+        $i = @qinverse + 1;
+        for my $inv (reverse @qinverse) {
+            @equation = grep { /q$i/ ? s/q$i/$inv/ : $_ } @equation;
+            $i--;
+        }
+
+        # Remove the 1-suffix from the equation.
+        @equation = grep { /p1/ ? s/p1/p/g : $_ } @equation;
+        @equation = grep { /q1/ ? s/q1/q/g : $_ } @equation;
+
+        # 
+        my $payoff = join ' + ', @equation;
+
+        # Create the expression.
+        my $exp = Math::Calculus::Differentiate->new;
+        $exp->addVariable('p');
+        $exp->addVariable('q');
+        $exp->setExpression($payoff) or die $exp->getError;
+        $exp->simplify or die $exp->getError;
+#warn "E: ",$exp->getExpression, "\n";
+        $exp->differentiate( $player eq 1 ? 'p' : 'q' ) or die $exp->getError;
+        $exp->simplify or die $exp->getError;
+#warn "D: ",$exp->getExpression, "\n";
+
+        # Set the player payoff strategy equation.
+        $self->{$player}{payoff} = $exp->getExpression;
+        push @payoff, $exp->getExpression;
+    }
+
+    return \@payoff;
+}
+
 1;
 
 __END__
@@ -168,17 +256,19 @@ Game::Theory::TwoPersonMatrix - Reduce & analyze a 2 person matrix game
 
 =head1 VERSION
 
-version 0.0101
+version 0.02
 
 =head1 SYNOPSIS
 
   use Game::Theory::TwoPersonMatrix;
   my $g = Game::Theory::TwoPersonMatrix->new(
-    1 => { 1 => \@u1, 2 => \@u2 },
-    2 => { 1 => \@v1, 2 => \@v2 }
+    1 => { strategy => { 1 => \@s1, 2 => \@s2, } },
+    2 => { strategy => { 1 => \@t1, 2 => \@t2, } },
   );
   $g->reduce(2, 1);
   $g->reduce(1, 2);
+  my $p = $g->payoff;
+  print Dumper $p;
   my $e = $g->nash;
   print Dumper $e;
 
@@ -190,11 +280,11 @@ of player names, strategies and numerical utilities.
 The players must have the same number of strategies, and each strategy must have
 the same size utility vectors as all the others.
 
-Player strategies are given by a 2D matrix of utilities (or payoffs) such that,
+Player strategies are given by a 2D matrix of utilities such that,
 
   [ [ u1, u2 .. un] .. [ v1, v2 .. vn ] ]
 
-Where each "B<u>i" is a utility or payoff for the strategy "B<U>."
+Where each "B<u>i" is a utility measure for the strategy "B<U>."
 
 Player 1 and 2 are the "row" and "column" players, respectively.  This is due to
 the tabular format of a matrix game:
@@ -210,6 +300,9 @@ The same game in "linear form" is:
  P1: { 1: [1,1], 3: [0,2] }
  P2: { 1: [0,2], 2: [3,4] }
 
+For a description of player probability profiles, please see the relevant
+literature.
+
 =head1 NAME
 
 Game::Theory::TwoPersonMatrix - Reduce & analyze a 2 person matrix game
@@ -227,6 +320,9 @@ Player defaults:
   1 => { 1 => [1,0], 2 => [0,1] }, # The "row player"
   2 => { 1 => [1,0], 2 => [0,1] }  # The "column player"
 
+If the probabilities are not given, they are computed to yield the same results
+as the player strategy profile.  W00!
+
 =head2 reduce()
 
   $g->reduce_game(1, 2); # Player 1 given opponent 2
@@ -247,6 +343,19 @@ insoluble.
 
 Find the Nash equilibria.
 
+=head2 payoff()
+
+  my $p = $g->payoff;
+  print Dumper $p;
+
+      | 0 3 0 |      | 3 0 0 |
+  A = | 2 1 3 |  B = | 1 2 3 |
+
+  PA = 0*p1*q1 + 3*p1*q2 + 0*p1*q3
+     + 2*p2*q1 + 1*p2*q2 + 3*p1*q3
+  PB = 3*p1*q1 + 0*p1*q2 ...
+     + 1*p2*q1 + 2*p2*q2
+
 =head1 AUTHOR
 
 Gene Boggs <gene@cpan.org>
